"""
Test modification to support typeguard.
"""

# Coverage is weird in this file - see
# https://pytest-cov.readthedocs.io/en/latest/plugins.html
# for environment variables we have to set to get coverage to work.

import inspect

import pytest
from typeguard import typechecked


def pytest_collection_modifyitems(items: list[pytest.Item]) -> None:
    """
    Apply the typeguard decorator to all collected test functions.
    """
    for item in items:
        if not isinstance(item, pytest.Function):
            continue
        if inspect.ismethod(item.obj) and isinstance(
            item.parent,
            pytest.Class,
        ):
            # If it's a method, we must handle the `self` argument
            # correctly.
            item.obj = typechecked(item.obj.__func__).__get__(  # pylint: disable=unnecessary-dunder-call
                item.parent.obj, item.parent.obj.__class__
            )
        else:
            # If it's a regular function, just apply the decorator.
            item.obj = typechecked(item.obj)
